/*
 * mocktcp
 *
 * This library provides a reliable, ordered, stream-based, message 
 * transport designed to look very similar to TCP, although slightly
 * cleaner. Underneath, mocktcp uses rmessage reliable datagrams for
 * packet delivery. Rmessages uses UDP.
 *
 * -Dave Lowell
 */

#ifndef MOCKTCP_H
#define MOCKTCP_H

#include "rmessage.h"

#define MT_MAX_MTU RM_MAX_MTU

typedef struct {
	char	buf[MT_MAX_MTU];	/* buffer */
	int	bytes;			/* number of bytes in buffer */
	int	index;			/* index of next avail bytes */
} conn_buffer;

extern void mt_init(void);
extern int mt_socket(void);
extern int mt_bind(int sock, int port);
extern int mt_listen(int sock);
extern int mt_accept(int sock, struct sockaddr_in *addr);
extern int mt_connect(int sock, char* host, int port);
extern int mt_send(int sock, void* msg, int len, int flags);
extern int mt_recv(int sock, void* buf, int len, int flags);
extern int mt_write(int sock, void* msg, int len);
extern int mt_read(int sock, void* buf, int len);

#endif

