
#include "rmessage.h"
#include <signal.h>

#define SIZE 8192
#define TIMES 10000

void my_handler(int s)
{
	fprintf(stderr, "\n\n############# INTR SIG ##############\n");
	print_trace();
	exit(0);
}

main(int argc, char** argv)
{
	int	s, port, res, i, bufsize, data;
	long	start, finish;
	double	delta;
	char	buf[SIZE];
	struct sockaddr_in client;
	struct sigaction sa;

	if (argc != 2) {
		fprintf(stderr, "usage: server {port}\n");
		exit(1);
	}

	port  = atoi(argv[1]);

	fprintf(stderr, "Server waiting on port %d.\n", port);

	sa.sa_handler = my_handler;
	sigemptyset(&sa.sa_mask);
	sa.sa_flags = 0;
	if (sigaction(SIGINT, &sa, NULL) < 0) {
		perror("sigaction");
		exit(1);
	}

	rm_init();
	s = rm_socket();
	if (s < 0) {
		perror("rm_socket");
		exit(1);
	}

	res = rm_bind(s, port);
	if (res < 0) {
		perror("rm_bind");
		exit(1);
	}

	bufsize = 240 * 1024;
	res = setsockopt(s, SOL_SOCKET, SO_RCVBUF, &bufsize, sizeof(int));
	if (res < 0) {
		perror("setsockopt");
		exit(1);
	}

	start = get_time();
	data = 0;

	for (i=0; i < TIMES; i++) {
		res = rm_recv(s, buf, SIZE, 0);
		if (res < 0) {
			perror("rm_recv");
			exit(1);
		}
		data += res;
	}
	
	finish = get_time();
	delta = elapsed_time(start, finish);

	sleep(1);
	rm_close(s);

	fprintf(stderr, "data == %d, delta = %.3f\n", data, delta);
	fprintf(stderr, "%.2f MB/sec\n", (data/(1024*1024))/delta);

}

